#ifndef _CURSOR_H_
#define _CURSOR_H_

#include <iostream.h>

struct SCursorSpec
{
  void reset( void ) 
  { 
    offset = 0;
    x1 = x2 = y = cell = maxCell = 0; 
  }

  void dump( void )
  {
    cout << "offset: " << offset << " y: " << y << " x1: " << x1 << endl;
    cout << "x2: " << x2 << " cell: " << cell << endl;
  }

  uint offset;
  int  y;
  int  x1;
  int  x2;
  int  cell;
  int  maxCell;
  char data;
};


struct SCursor
{
  SCursor( void )
  {
    mTextWidth = 1;
    mArea = 0;
  }

  void reset( void )
  {
    prev.reset();
    curr.reset();
    next.reset();
  }

  bool incCell( void )
  {
    if( curr.cell + 1 >= curr.maxCell )
    {
      addOffset( 1 );
      return( true );
    }
    else
    {
      next.cell = curr.cell + 1;
      return( false );
    }
  }

  bool decCell( void )
  {
    if( curr.cell == 0 )
    {
      if( curr.offset > 0 )
      {
	decOffset( 1, true );
	next.cell = curr.maxCell > 0 ? curr.maxCell - 1 : 0;
      }
      return( true );
    }
    else
    {
      next.cell = curr.cell - 1;
      return( false );
    }
  }


  void addOffset( uint val )
  {
    setOffset( curr.offset + val );
  }

  void decOffset( uint val, bool ignoreCell )
  {
    if( ignoreCell == true || curr.cell == 0 )
    {
      setOffset( val > curr.offset ? curr.offset%mTextWidth : curr.offset-val);
    }
    else
    {
      setOffset( curr.offset );
    }
  }

  void setOffset( uint offset )
  {
    next.offset = offset;
    next.cell = 0;
  }

  void setOffset( uint offset, uint bit, bool backward, bool fromCursor,
		  uint textSize )
  {
    if( fromCursor == true )
    {
      if( backward == true )
      {
	setOffset( offset > curr.offset ? 0 : curr.offset - offset );
      }
      else
      {
	setOffset( curr.offset + offset );
      }
    }
    else
    {
      if( backward == true )
      {
	setOffset( offset > textSize ? 0 : textSize - offset );
      }
      else
      {
	setOffset( offset );
      }
    } 
    setBit( bit );
  }


  void setBit( uint bit )
  {
    bit = (bit > 7) ? 0 : 7 - bit;
    next.cell = bit / mCellWeight;
  }


  void home( bool toExtreme )
  {
    if( toExtreme == true )
    {
      setOffset( 0 );
    }
    else
    {
      setOffset( next.offset - curr.offset % mTextWidth );
    }
  }

  void end( bool toExtreme )
  {
    if( toExtreme == true )
    {
      setOffset( 0xFFFFFFFF );
    }
    else
    {
      setOffset( next.offset + (mTextWidth-1) - curr.offset % mTextWidth );
    }

  }

  void up( uint numLines )
  {
    decOffset( numLines * mTextWidth, true );
  }

  void down( uint numLines )
  {
    addOffset( numLines * mTextWidth );
  }

  bool changed( void )
  {
    return( prev.offset!=curr.offset || prev.cell!=curr.cell ? true : false );
  }

  uint bit( void )
  {
    uint bitValue = mCellWeight*(curr.maxCell - curr.cell) - 1;
    return( bitValue > 7 ? 7 : bitValue );
  }

  void setTextWidth( uint textWidth )
  {
    mTextWidth = textWidth < 1 ? 1 : textWidth;
  }

  void setCellWeight( uint cellWeight )
  {
    mCellWeight = cellWeight;
  }

  uint area( void )
  {
    return( mArea );
  }

  void setArea( uint area )
  {
    mArea = area;
  }
  
  uint mTextWidth;
  uint mCellWeight;
  uint mArea;
  SCursorSpec prev;
  SCursorSpec curr;
  SCursorSpec next;
};


#endif
