#include "hextoolwidget.h"
#include "kapp.h"

CHexToolWidget::CHexToolWidget( QWidget *parent, const char *name )
  : KTabCtl( parent, name )
{
  mConversion.page = new QWidget( this, "Conversion" );
  if( mConversion.page == 0 ) { return; }
  mCalculator.page = new QWidget( this, "Find" );
  if( mCalculator.page == 0 ) { return; }

  addTab( mConversion.page, "Conversion" ); 
  addTab( mCalculator.page, "Calculator" );

  CreateConversionPage();
  mCursorState.valid = false;

  connect( kapp, SIGNAL( kdisplayPaletteChanged() ),
	   this, SLOT( paletteChanged() ) );

  show();
}


CHexToolWidget::~CHexToolWidget( void )
{
  delete mConversion.page;
  delete mCalculator.page;
}




#define MAX(x,y) ((x) > (y) ? (x): (y))

void CHexToolWidget::CreateConversionPage( void )
{
  mUtilBox = new QGridLayout( mConversion.page, 5, 6, 5 );

  char *msg1[4]={ "Signed 8 bit", "Unsigned 8 bit", "Signed 16 bit", 
		  "Unsigned 16 bit" };
  char *msg2[4]={ "Signed 32 bit", "Unsigned 32 bit", "32 bit float", 
		  "64 bit float" };
  char *msg3[4]={ "Hexadecimal", "Octal", "Binary", "Text" };

  QLabel *label1[4];
  QLabel *label2[4];
  QLabel *label3[4];

  int max1 = 0;
  int max2 = 0;
  int max3 = 0;
  for( uint i=0; i<4; i++ )
  {
    label1[i] = new QLabel( i18n( msg1[i] ), mConversion.page );
    label1[i]->setMinimumSize( label1[i]->sizeHint() );
    max1 = MAX( max1, label1[i]->sizeHint().width() );
    mUtilBox->addWidget( label1[i], i, 0 );
    label1[i]->setAlignment( AlignRight|AlignVCenter );

    mText1[i] = new QLabel( "XXXXXXXX", mConversion.page );
    mText1[i]->setMinimumSize( mText1[i]->sizeHint() );
    mText1[i]->setMaximumWidth( mText1[i]->sizeHint().width() );
    mUtilBox->addWidget( mText1[i], i, 1 );

    label2[i] = new QLabel( i18n( msg2[i] ), mConversion.page );
    label2[i]->setMinimumSize( label2[i]->sizeHint() );
    max2 = MAX( max2, label2[i]->sizeHint().width() );
    mUtilBox->addWidget( label2[i], i, 2 );
    label2[i]->setAlignment( AlignRight|AlignVCenter );

    mText2[i] = new QLabel( "XXXXXXXXXXXX", mConversion.page );
    mText2[i]->setMinimumSize( mText2[i]->sizeHint() );
    mText2[i]->setMaximumWidth( mText2[i]->sizeHint().width() );
    mUtilBox->addWidget( mText2[i], i, 3 );

    label3[i] = new QLabel( i18n( msg3[i] ), mConversion.page );
    label3[i]->setMinimumSize( label3[i]->sizeHint() );
    max3 = MAX( max1, label3[i]->sizeHint().width() );
    mUtilBox->addWidget( label3[i], i, 4 );
    label3[i]->setAlignment( AlignRight|AlignVCenter );

    mText3[i] = new QLabel( "XXXXXXXX", mConversion.page );
    mText3[i]->setMinimumSize( mText3[i]->sizeHint() );
    mText3[i]->setMaximumWidth( mText3[i]->sizeHint().width() );
    mUtilBox->addWidget( mText3[i], i, 5 );
  }

  mCheckIntelFormat = new QCheckBox( 
    i18n("Use little endian decoding"), mConversion.page );
  mCheckIntelFormat->setMinimumSize( mCheckIntelFormat->sizeHint() );
  connect( mCheckIntelFormat, SIGNAL(clicked()),this,SLOT(intelFormat()) );
  mCheckIntelFormat->setChecked( true );
  mUtilBox->addWidget( mCheckIntelFormat, 4, 0 );

  mCheckHexadecimal = new QCheckBox( 
    i18n("Display unsigned numbers as hexadecimal"), mConversion.page );
  mCheckHexadecimal->setMinimumSize( mCheckHexadecimal->sizeHint() );
  connect( mCheckHexadecimal, SIGNAL(clicked()),this,SLOT(unsignedFormat()) );
  mUtilBox->addWidget( mCheckHexadecimal, 4, 3 );

 
  for( uint i=0; i<4; i++ )
  {
    label1[i]->setMaximumWidth( max1 + 5 );
    label2[i]->setMaximumWidth( max2 + 5 );
    label3[i]->setMaximumWidth( max3 + 5 );
    mText1[i]->setLineWidth( 1 );
    mText2[i]->setLineWidth( 1 );
    mText3[i]->setLineWidth( 1 );
    mText1[i]->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    mText2[i]->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    mText3[i]->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    mText1[i]->setBackgroundColor( kapp->windowColor );
    mText2[i]->setBackgroundColor( kapp->windowColor );
    mText3[i]->setBackgroundColor( kapp->windowColor );    
    mText1[i]->setAlignment( AlignRight|AlignVCenter );
    mText2[i]->setAlignment( AlignRight|AlignVCenter );
    mText3[i]->setAlignment( AlignRight|AlignVCenter );
    mText1[i]->setMargin( 3 );
    mText2[i]->setMargin( 3 );
    mText3[i]->setMargin( 3 );
    mText1[i]->setText( "" );
    mText2[i]->setText( "" );
    mText3[i]->setText( "" );
  }

  mUtilBox->activate();
  mUtilBox->freeze();

}


void swapByte( char *data, uint size )
{
  for( uint i=0; i<size/2; i++ )
  {
    char tmp = data[i];
    data[i] = data[size-1-i];
    data[size-1-i] = tmp;
  }
  for( uint i=0; i<size; i++ )
  {
    printf("0x%02X ",  (unsigned char)data[i] );
  }
  puts("");

}



void CHexToolWidget::cursorChanged( SCursorState &state )
{
  if( state.valid == true )
  {
    char buf[40];
    if( mCheckIntelFormat->isChecked() == true )
    {
      //
      // Assume Little Endian. This is the case for the Intel architecture. 
      // The 'state.data' stores the first byte from file/memory in its 
      // first array element.
      //

      unsigned int i,j,k;
      unsigned long long d;
      float fv;
      double dv;

      i = state.data[5]<<8 | state.data[4];
      j = state.data[7]<<8 | state.data[6];
      d = i | j<<16;
      
      i = state.data[1]<<8 | state.data[0];
      j = state.data[3]<<8 | state.data[2];
      k = i | j<<16;
      d = d<<32 | k;

      memcpy( &fv, &k, 4 );
      memcpy( &dv, &d, 8 );

      if( mCheckHexadecimal->isChecked() == true )
      {
	sprintf( buf, "%d", (char)state.data[0] );
	mText1[0]->setText( buf );
	sprintf( buf, "%02X", state.data[0] );
	mText1[1]->setText( buf );
	sprintf( buf, "%d", (short)i );
	mText1[2]->setText( buf );
	sprintf( buf, "%04X", (unsigned short)i );
	mText1[3]->setText( buf );
	sprintf( buf, "%d", (int)k );
	mText2[0]->setText( buf );
	sprintf( buf, "%08X", k );
	mText2[1]->setText( buf );
	sprintf( buf, "%E", fv );
	mText2[2]->setText( buf );
	sprintf( buf, "%E", dv );
	mText2[3]->setText( buf );
      }
      else
      {
	sprintf( buf, "%d", (char)state.data[0] );
	mText1[0]->setText( buf );
	sprintf( buf, "%u", state.data[0] );
	mText1[1]->setText( buf );
	sprintf( buf, "%d", (short)i );
	mText1[2]->setText( buf );
	sprintf( buf, "%u", (unsigned short)i );
	mText1[3]->setText( buf );
	sprintf( buf, "%d", (int)k );
	mText2[0]->setText( buf );
	sprintf( buf, "%u", k );
	mText2[1]->setText( buf );
	sprintf( buf, "%E", fv );
	mText2[2]->setText( buf );
	sprintf( buf, "%E", dv );
	mText2[3]->setText( buf );
      }
    }
    else
    {
      //
      // Assume Big Endian. This is the case for SUN machines (amongst others)
      // The 'state.data' stores the first byte from file/memory in its 
      // first array element.
      //
      unsigned int i,j,k;
      unsigned long long d;
      float fv;
      double dv;

      i = state.data[5] | state.data[4]<<8;
      j = state.data[7] | state.data[6]<<8;
      d = i<<16 | j;
      
      i = state.data[1] | state.data[0]<<8;
      j = state.data[3] | state.data[2]<<8;
      k = i<<16 | j;
      d = d<<32 | k;

      memcpy( &fv, &k, 4 );
      memcpy( &dv, &d, 8 );
      
      if( mCheckHexadecimal->isChecked() == true )
      {
	sprintf( buf, "%d", (char)state.data[0] );
	mText1[0]->setText( buf );
	sprintf( buf, "%02X", state.data[0] );
	mText1[1]->setText( buf );
	sprintf( buf, "%d", (short)i );
	mText1[2]->setText( buf );
	sprintf( buf, "%04X", (unsigned short)i );
	mText1[3]->setText( buf );
	sprintf( buf, "%d", (int)k );
	mText2[0]->setText( buf );
	sprintf( buf, "%08X", k );
	mText2[1]->setText( buf );
	sprintf( buf, "%E", fv );
	mText2[2]->setText( buf );
	sprintf( buf, "%E", dv );
	mText2[3]->setText( buf );
      }
      else
      {
	sprintf( buf, "%d", (char)state.data[0] );
	mText1[0]->setText( buf );
	sprintf( buf, "%u", state.data[0] );
	mText1[1]->setText( buf );
	sprintf( buf, "%d", (short)i );
	mText1[2]->setText( buf );
	sprintf( buf, "%u", (unsigned short)i );
	mText1[3]->setText( buf );
	sprintf( buf, "%d", (int)k );
	mText2[0]->setText( buf );
	sprintf( buf, "%u", k );
	mText2[1]->setText( buf );
	sprintf( buf, "%E", fv );
	mText2[2]->setText( buf );
	sprintf( buf, "%E", dv );
	mText2[3]->setText( buf );
      }
    }

    unsigned char data = (unsigned char)state.data[0];
    sprintf( buf, "%02X", data );
    mText3[0]->setText( buf );
    sprintf( buf, "%03o", data );
    mText3[1]->setText( buf );
    for( int i = 0; i < 8; i++ )
    {
      buf[7-i] = (data&(1<<i)) ? '1' : '0';
    }
    buf[8] = 0;
    mText3[2]->setText( buf );
    sprintf( buf, "%c", data < 32 ? '.' : data );
    mText3[3]->setText( buf );
  }
  else
  {
    for( int i=0; i<4; i++)
    {
      mText1[i]->setText( "" );
      mText2[i]->setText( "" );
      mText3[i]->setText( "" );
    }
  }

  mCursorState = state;
}



void CHexToolWidget::paletteChanged( void )
{
  for( int i=0; i<4; i++ )
  {
    mText1[i]->setBackgroundColor( kapp->windowColor );
    mText2[i]->setBackgroundColor( kapp->windowColor );
    mText3[i]->setBackgroundColor( kapp->windowColor );  
  }
}

void CHexToolWidget::intelFormat( void )
{
  cursorChanged( mCursorState );
}


void CHexToolWidget::unsignedFormat( void )
{
  cursorChanged( mCursorState );
}


void CHexToolWidget::resizeEvent( QResizeEvent *e )
{
}




#if 0
void CHexToolWidget::cursorChanged( SCursorState &state )
{
  if( state.valid == true )
  {
    char buf[40];
    if( mCheckIntelFormat->isChecked() == true )
    {
      union number
      {
	char c;
	short s;
	int i;
	float f;
	double d;
      };

      number x;
      memcpy( &x, &state.data, 8 );

      sprintf( buf, "%d", x.c );
      mText1[0]->setText( buf );
      sprintf( buf, "%u", (unsigned char)x.c );
      mText1[1]->setText( buf );
      sprintf( buf, "%d", x.s );
      mText1[2]->setText( buf );
      sprintf( buf, "%u", (unsigned short)x.s );
      mText1[3]->setText( buf );
      sprintf( buf, "%d", x.i );
      mText2[0]->setText( buf );
      sprintf( buf, "%u", (unsigned int)x.i );
      mText2[1]->setText( buf );
      sprintf( buf, "%E", x.f );
      mText2[2]->setText( buf );
      sprintf( buf, "%E", x.d );
      mText2[3]->setText( buf );
    }

    #if 0
    if( mCheckIntelFormat->isChecked() == true )
    {
      sprintf( buf, "%d", *(char*)&state.data );
      mText1[0]->setText( buf );
      sprintf( buf, "%u", *(unsigned char*)&state.data );
      mText1[1]->setText( buf );
      sprintf( buf, "%d", *(short*)&state.data );
      mText1[2]->setText( buf );
      sprintf( buf, "%u", *(unsigned short*)&state.data );
      mText1[3]->setText( buf );
    
      sprintf( buf, "%d", *(int*)&state.data );
      mText2[0]->setText( buf );
      sprintf( buf, "%u", *(unsigned int*)&state.data );
      mText2[1]->setText( buf );
      
      /*
      float f1=(float)*(float*)&state.data;
      float f2=(float)(*(unsigned int*)&state.data);
      unsigned int u1=(*(unsigned int*)&state.data);
      printf( "1) %E\n", f1 );
      printf( "2) %E\n", f2 );
      uchar *c1 = (uchar*)&f1;
      uchar *c2 = (uchar*)&f2;
      uchar *c3 = (uchar*)&state.data;
      uchar *c4 = (uchar*)&u1;

      for( int i=0; i<4; i++ )
      {
	printf("(%02X,%02X,%02X,%02X) ", c1[i], c2[i], c3[i], c4[i] );
      }
      puts("");
      */

      union number
      {
	char c;
	short s;
	int i;
	float f;
	double d;
      };

      number x;
      memcpy( &x, &state.data, 8 );
      
      printf( "float %E\n", x.f );
      printf( "double %E\n", x.d );


      /*
      float f1=(float)*(float*)&state.data;
      float f2=(float)*(unsigned int*)&state.data;
      uchar *c1 = (uchar*)&f1;
      uchar *c2 = (uchar*)&f2;
      for( int i=0; i<4; i++ )
      {
	printf("(%02X,%02X) ", c1[i], c2[i] );
      }
      puts("");
      */

      float f;
      memcpy( &f, &state.data, 4 ); 
      sprintf( buf, "%E", f );
      mText2[2]->setText( buf );

      





      //sprintf( buf, "%E", (float)(*(unsigned int*)&state.data) );
      //mText2[2]->setText( buf );

      double d;
      memcpy( &d, &state.data, 8 ); 
      sprintf( buf, "%E", d );
      mText2[3]->setText( buf );
    }
    #endif

    else
    {
      sprintf( buf, "%d", *(char*)&state.data );
      mText1[0]->setText( buf );
      sprintf( buf, "%u", *(unsigned char*)&state.data );
      mText1[1]->setText( buf );
      
      short s = *(short*)&state.data;
      swapByte( (char*)&s, sizeof(short) );

      sprintf( buf, "%d", s );
      mText1[2]->setText( buf );
      sprintf( buf, "%u", (unsigned short)s );
      mText1[3]->setText( buf );

      int i = *(int*)&state.data;
      swapByte( (char*)&i, sizeof(int) );
    
      sprintf( buf, "%d", i );
      mText2[0]->setText( buf );
      sprintf( buf, "%u", (unsigned int)i );
      mText2[1]->setText( buf );
      sprintf( buf, "%E", *(float*)&state.data );
      mText2[2]->setText( buf );
      sprintf( buf, "%E", *(double*)&state.data );
      mText2[3]->setText( buf );
    }

    unsigned char data = *(unsigned char*)&state.data;
    sprintf( buf, "%02X", data );
    mText3[0]->setText( buf );
    sprintf( buf, "%03o", data );
    mText3[1]->setText( buf );
    for( int i = 0; i < 8; i++ )
    {
      buf[7-i] = (data&(1<<i)) ? '1' : '0';
    }
    buf[8] = 0;
    mText3[2]->setText( buf );
    sprintf( buf, "%c", data < 32 ? '.' : data );
    mText3[3]->setText( buf );
  }
  else
  {
    for( int i=0; i<4; i++)
    {
      mText1[i]->setText( "" );
      mText2[i]->setText( "" );
      mText3[i]->setText( "" );
    }
  }

  mCursorState = state;
}
#endif




#if 0
void CHexToolWidget::cursorChanged( SCursorState &state )
{

  /*
  unsigned char buf[4] = { 1, 2, 3, 4 };
  float f = *(float*)&buf;
  unsigned char *c = (unsigned char*)&f;

  for( int i=0; i<4;i++)
  {
    printf("(%02X,%02X) ", buf[i], c[i] );
  }
  puts("");
  */
  




  if( state.valid == true )
  {
    char buf[40];
    if( mCheckIntelFormat->isChecked() == true )
    {
      //
      // Show as little endian. The 'data' is stored in "little endian
      // mode" (first byte from file in first location of buffer) so a
      // simple memcpy() should do the job.
      //
      union number
      {
	char c;
	short s;
	int i;
	float f;
	double d;
      } x;
      memcpy( &x, state.data, sizeof(state.data) );

      sprintf( buf, "%d", x.c );
      mText1[0]->setText( buf );
      sprintf( buf, "%u", (unsigned char)x.c );
      mText1[1]->setText( buf );
      sprintf( buf, "%d", x.s );
      mText1[2]->setText( buf );
      sprintf( buf, "%u", (unsigned short)x.s );
      mText1[3]->setText( buf );
      sprintf( buf, "%d", x.i );
      mText2[0]->setText( buf );
      sprintf( buf, "%u", (unsigned int)x.i );
      mText2[1]->setText( buf );
      sprintf( buf, "%E", x.f );
      mText2[2]->setText( buf );
      sprintf( buf, "%E", x.d );
      mText2[3]->setText( buf );
    }
    else
    {
      union number
      {
	char c;
	short s;
	int i;
	float f;
	double d;
      };

      number x;
      //memcpy( &x, &state.data, sizeof(state.data) );
      memcpy( &x, state.data, sizeof(state.data) );
      swapByte( (char*)&x, sizeof(state.data) );

      sprintf( buf, "%d", x.c );
      mText1[0]->setText( buf );
      sprintf( buf, "%u", (unsigned char)x.c );
      mText1[1]->setText( buf );
      sprintf( buf, "%d", x.s );
      mText1[2]->setText( buf );
      sprintf( buf, "%u", (unsigned short)x.s );
      mText1[3]->setText( buf );
      sprintf( buf, "%d", x.i );
      mText2[0]->setText( buf );
      sprintf( buf, "%u", (unsigned int)x.i );
      mText2[1]->setText( buf );
      sprintf( buf, "%E", x.f );
      mText2[2]->setText( buf );
      sprintf( buf, "%E", x.d );
      mText2[3]->setText( buf );
    }

    //unsigned char data = *(unsigned char*)&state.data;
    unsigned char data = (unsigned char)state.data[0];
    sprintf( buf, "%02X", data );
    mText3[0]->setText( buf );
    sprintf( buf, "%03o", data );
    mText3[1]->setText( buf );
    for( int i = 0; i < 8; i++ )
    {
      buf[7-i] = (data&(1<<i)) ? '1' : '0';
    }
    buf[8] = 0;
    mText3[2]->setText( buf );
    sprintf( buf, "%c", data < 32 ? '.' : data );
    mText3[3]->setText( buf );
  }
  else
  {
    for( int i=0; i<4; i++)
    {
      mText1[i]->setText( "" );
      mText2[i]->setText( "" );
      mText3[i]->setText( "" );
    }
  }

  mCursorState = state;
}
#endif



#if 0
void CHexToolWidget::cursorChanged( SCursorState &state )
{
  if( state.valid == true )
  {
    char buf[40];
    if( mCheckIntelFormat->isChecked() == true )
    {
      //
      // Show as little endian. The 'data' is stored in "little endian
      // mode" (first byte from file in first location of buffer) so a
      // simple memcpy() should do the job here.
      //
      /*
      union number
      {
	char c;
	short s;
	int i;
	float f;
	double d;
      } x;
      memcpy( &x, state.data, sizeof(state.data) );
      */

      unsigned int i,j,k;
      unsigned long long d;
      float fv;
      double dv;

      i = state.data[5]<<8 | state.data[4];
      j = state.data[7]<<8 | state.data[6];
      d = i | j<<16;
      
      i = state.data[1]<<8 | state.data[0];
      j = state.data[3]<<8 | state.data[2];
      k = i | j<<16;
      d = d<<32 | k;

      memcpy( &fv, &k, 4 );
      memcpy( &dv, &d, 8 );
      

      sprintf( buf, "%d", (char)state.data[0] );
      mText1[0]->setText( buf );
      sprintf( buf, "%u", state.data[0] );
      mText1[1]->setText( buf );
      sprintf( buf, "%d", (short)i );
      mText1[2]->setText( buf );
      sprintf( buf, "%u", (unsigned short)i );
      mText1[3]->setText( buf );
      sprintf( buf, "%d", (int)k );
      mText2[0]->setText( buf );
      sprintf( buf, "%u", k );
      mText2[1]->setText( buf );
      sprintf( buf, "%E", fv );
      mText2[2]->setText( buf );
      sprintf( buf, "%E", dv );
      mText2[3]->setText( buf );




      /*
      sprintf( buf, "%d", x.c );
      mText1[0]->setText( buf );
      sprintf( buf, "%u", (unsigned char)x.c );
      mText1[1]->setText( buf );
      sprintf( buf, "%d", x.s );
      mText1[2]->setText( buf );
      sprintf( buf, "%u", (unsigned short)x.s );
      mText1[3]->setText( buf );
      sprintf( buf, "%d", x.i );
      mText2[0]->setText( buf );
      sprintf( buf, "%u", (unsigned int)x.i );
      mText2[1]->setText( buf );
      sprintf( buf, "%E", x.f );
      mText2[2]->setText( buf );
      sprintf( buf, "%E", x.d );
      mText2[3]->setText( buf );
      */
    }
    else
    {
      //
      // Show as big endian. The 'data' is stored in "little endian
      // mode" (first byte from file in first location of buffer) so we
      // have to do the swapping.
      //
      unsigned int i,j,k;
      unsigned long long d;
      double dT;

      i = state.data[5] | state.data[4]<<8;
      j = state.data[7] | state.data[6]<<8;
      d = j | i<<16;
      
      i = state.data[1] | state.data[0]<<8;
      j = state.data[3] | state.data[2]<<8;
      k = j | i<<16;
      d = d<<32 | k;

      memcpy( &dT, &d, 8 );

      sprintf( buf, "%d", (char)state.data[0] );
      mText1[0]->setText( buf );
      sprintf( buf, "%u", state.data[0] );
      mText1[1]->setText( buf );
      sprintf( buf, "%d", (short)i );
      mText1[2]->setText( buf );
      sprintf( buf, "%u", (unsigned short)i );
      mText1[3]->setText( buf );
      sprintf( buf, "%d", (int)k );
      mText2[0]->setText( buf );
      sprintf( buf, "%u", k );
      mText2[1]->setText( buf );
      sprintf( buf, "%E", (float)k );
      mText2[2]->setText( buf );
      sprintf( buf, "%E", /*(double)*/dT );
      mText2[3]->setText( buf );
    }

    unsigned char data = (unsigned char)state.data[0];
    sprintf( buf, "%02X", data );
    mText3[0]->setText( buf );
    sprintf( buf, "%03o", data );
    mText3[1]->setText( buf );
    for( int i = 0; i < 8; i++ )
    {
      buf[7-i] = (data&(1<<i)) ? '1' : '0';
    }
    buf[8] = 0;
    mText3[2]->setText( buf );
    sprintf( buf, "%c", data < 32 ? '.' : data );
    mText3[3]->setText( buf );
  }
  else
  {
    for( int i=0; i<4; i++)
    {
      mText1[i]->setText( "" );
      mText2[i]->setText( "" );
      mText3[i]->setText( "" );
    }
  }

  mCursorState = state;
}
#endif

