#include <iostream.h>

#include <qmsgbox.h> 
#include <kapp.h>
#include <kiconloader.h>
#include <kstdaccel.h>

#include "toplevel.h"

static char *blurb = "Hex Editor 0.7 - Extended hex editor for KDE.\n\n"
                     "Author: Espen Sand\n" 
                     "espensa@online.no\n\n"
                     "This is a work in progress. If you\n\n"
                     "1) have any suggestions for improvement\n"
                     "2) have found a bug\n"
                     "3) just want to tell me how nice it is\n\n"
                     "then feel free to send me a mail.\n\n"
                     "Latest version is in KDE CVS and on\n"
                     "http://home.sol.no/~espensa/khexedit.";





QList<KHexEdit> KHexEdit::mWindowList;


#define Xview_hexadecimal       0
#define Xview_octal             1
#define Xview_binary            2
#define Xview_text              3
#define Xview_offset_area       4
#define Xview_text_area         5
#define Xview_offset_as_decimal 6
#define Xview_case_data         7
#define Xview_case_offset       8


#define option_toolbar      0
#define option_statusbar    1
#define option_utility      2
#define option_edit         3

#define toolbarCopy         1
#define toolbarCut          2
#define toolbarUndo         3
#define toolbarRedo         4
#define toolbarInternetStop 5

KHexEdit::KHexEdit( void )
{
  mWindowList.append( this );
  mDocumentList.setAutoDelete( true );
  mValid = false;

  /*
  mShowHexOffset    = true;
  mShowToolBar      = true;
  mShowStatusBar    = true;
  mShowUtility      = true;
  */

  //
  // Create main widget and register it.
  //
  mManager = new CHexManagerWidget( this, "manager" );
  if( mManager == 0 ) { return; }
  setView( mManager );


  connect( textView(), SIGNAL( cursorChanged( SCursorState & ) ),
	   this, SLOT( cursorChanged( SCursorState & ) ) );
  connect( textView(), SIGNAL( editMode( CHexBuffer::EEditMode ) ),
	   this, SLOT( editMode( CHexBuffer::EEditMode ) ) );
  connect( textView(), SIGNAL( textWidth( uint ) ),
	   this, SLOT( textWidth( uint ) ) );
  connect( textView(), SIGNAL( fileModified( SFileState & ) ),
	   this, SLOT( fileModified( SFileState & ) ) );
  connect( textView(), SIGNAL( displayChanged( const SDisplayState & ) ),
	   this, SLOT( displayChanged( const SDisplayState & ) ) );
  connect( textView(), SIGNAL( fileName( QString & ) ),
	   this, SLOT( fileActive( QString & ) ) );
  connect( textView(), SIGNAL( fileClosed( QString & ) ),
	   this, SLOT( fileClosed( QString & ) ) ); 
  connect( editor(), SIGNAL( internetLoadingChanged( bool ) ),
	   this, SLOT( internetLoadingChanged( bool ) ) );

  //
  // The entire window will act as a drop zone (gee, this is really 
  // cumbersome in Motif)
  //
  mZone = new KDNDDropZone( this , DndURL );
  if( mZone == 0 ) { return; }
  connect( mZone, SIGNAL( dropAction(KDNDDropZone *) ), 
	   this, SLOT( dropAction(KDNDDropZone *) ) );

  //
  // Make the menu bar entries. We use the default entry for the help menu.
  //
  mRecent = new QPopupMenu;
  if( mRecent == 0 ) { return; }
  connect( mRecent, SIGNAL(activated(int)), SLOT(recentFile(int)) );

  mFile = new QPopupMenu;
  if( mFile == 0 ) { return; }

  mEditSpecial = new QPopupMenu;
  if( mEditSpecial == 0 ) { return; }

  mEdit = new QPopupMenu;
  if( mEdit == 0 ) { return; }

  mView = new QPopupMenu;
  if( mView == 0 ) { return; }

  mDocuments = new QPopupMenu;
  if( mDocuments == 0 ) { return; }
  connect( mDocuments, SIGNAL(activated(int)), SLOT(selectDocument(int)) );

  mTools = new QPopupMenu;
  if( mTools == 0 ) { return; }

  mOptions = new QPopupMenu;
  if( mOptions == 0 ) { return; }

  mHelp = kapp->getHelpMenu( true, blurb );

  //
  // Make the contents of the menus a connect the entries to a slot
  //
  KStdAccel *keys = new KStdAccel( kapp->getConfig() );
  if( keys == 0 ) { return; }

  mFile->insertItem( i18n("&New File"), 
		     editor(), SLOT(newFile()), keys->openNew() );
  mFile->insertItem( i18n("&Open..."), 
		     editor(), SLOT(open()), keys->open() );
  mFile->insertItem( i18n("&Insert..."), 
		     editor(), SLOT(insertFile()), CTRL+Key_I );
  mFile->insertItem( i18n("Open R&ecent"), mRecent );
  mFile->insertItem( i18n("&Reload"), 
		     editor(), SLOT(reload()) );
  mFile->insertSeparator();
  mFile->insertItem( i18n("&Save"), 
		     editor(), SLOT(save()), keys->save() );
  mFile->insertItem( i18n("Save &as..."), 
		     editor(), SLOT(saveAs()), CTRL+Key_A );
  mFile->insertItem( i18n("&Close File"), 
		     editor(), SLOT(close()), keys->close() );
  mFile->insertSeparator();
  mFile->insertItem( i18n("&Print..."), 
		     editor(), SLOT(print()), keys->print() );
  mFile->insertItem( i18n("E&xport..."), 
		     editor(), SLOT(export()) );
  mFile->insertSeparator();
  mFile->insertItem( i18n("N&ew Window"), 
		     this, SLOT(newWindow()) );
  mFile->insertItem( i18n("Close &Window"), 
		     this, SLOT(closeWindow()) );
  mFile->insertSeparator();
  mFile->insertItem( i18n("&Quit"), 
		     this, SLOT(closeProgram()), keys->quit() );

  mEditUndoMB = mEdit->insertItem( i18n("&Undo"), 
		     editor(), SLOT(undo()), keys->undo() );
  mEditRedoMB = mEdit->insertItem( i18n("&Redo"), 
		     editor(), SLOT(redo()), CTRL+Key_Y );
  mEdit->insertSeparator();
  mEditCopyMB = mEdit->insertItem( i18n("&Copy"),
		     editor(), SLOT(copy()), keys->copy() );
  mEdit->insertItem( i18n("&Paste"),
		     editor(), SLOT(paste()), keys->paste() );
  mEditCutMB = mEdit->insertItem( i18n("Cu&t"),
		     editor(), SLOT(cut()), keys->cut() );
  mEdit->insertItem( i18n("Spec&ial"), mEditSpecial );
  mEdit->insertSeparator();
  mEdit->insertItem( i18n("&Select all"), 
		     editor(), SLOT(selectAll()) );
  mEdit->insertItem( i18n("&Unselect"), 
		     editor(), SLOT(unselect()) );
  mEdit->insertSeparator();
  mEdit->insertItem( i18n("&Goto..."),
		     editor(), SLOT(gotoOffset()), CTRL+Key_G );
  mEdit->insertItem( i18n("&Find..."),
		     editor(), SLOT(find()), keys->find() );
  mEdit->insertItem( i18n("Find &Again"),
		     editor(), SLOT(findAgain()), Key_F3 );
  mEdit->insertItem( i18n("Find and R&eplace..."),
		     editor(), SLOT(replace()), keys->replace() );
  mEdit->insertItem( i18n("Insert pattern..."),
		     editor(), SLOT(insertPattern()), CTRL+Key_Insert );

  mEditSpecialCopyText1MB = mEditSpecial->insertItem( 
			    i18n("Copy as text (&primary field)"),
			    editor(), SLOT(copyPrimaryFormatted()) );
  mEditSpecialCopyText2MB = mEditSpecial->insertItem( 
			    i18n("Copy as text (&all fields)"),
			    editor(), SLOT(copyAllFormatted()) );
  mEditSpecial->insertSeparator();
  mEditSpecial->insertItem( i18n("Paste into new &file"),
			    editor(), SLOT(pasteNewFile()) );
  mEditSpecial->insertItem( i18n("Paste into new &window"),
			    this, SLOT(pasteNewWindow()) );


  mView->setCheckable( TRUE );
  mViewHexMB      = mView->insertItem( i18n("&Hexadecimal"), 
		     Xview_hexadecimal );
  mViewOctalMB    = mView->insertItem( i18n("&Octal"), 
		     Xview_octal );
  mViewBinaryMB   = mView->insertItem( i18n("&Binary"), 
		     Xview_binary );
  mViewTextOnlyMB = mView->insertItem( i18n("&Text only"), 
		     Xview_text );
  mView->insertSeparator();
  mViewOffsetColumnMB = mView->insertItem( i18n("Show o&ffset column"), 
		     Xview_offset_area );
  mViewTextAreaMB = mView->insertItem( i18n("Show t&ext area"), 
		     Xview_text_area );
  mView->insertSeparator();
  mViewOffsetAsDecimalMB = mView->insertItem( i18n("Offset as &decimal"), 
		     Xview_offset_as_decimal );
  mView->insertSeparator();
  mViewBigCaseDataMB = mView->insertItem( i18n("B&ig case (data)"), 
		     Xview_case_data );
  mViewBigCaseOffsetMB = mView->insertItem( i18n("Bi&g case (offset)"), 
		     Xview_case_offset );

  connect( mView, SIGNAL(activated (int)), SLOT( viewMenuCB (int) ));


  mTools->insertItem( i18n("&Extract strings..."),
		      0 );
  mTools->insertItem( i18n("&Statistics..."),
		      0 );


  mOptions->setCheckable( TRUE );

  mOptions->insertItem( i18n("&General preferences..."),
		        option_edit );
  mOptions->insertSeparator();
  mOptionToolBarMB   = mOptions->insertItem( i18n("Show &Tool Bar"), 
		        option_toolbar );
  mOptionStatusBarMB = mOptions->insertItem( i18n("Show &Status Bar"), 
		        option_statusbar );
  mOptionUtilityMB   = mOptions->insertItem( i18n("Show &Utilities"), 
		        option_utility );
  mOptions->insertSeparator();
  mOptions->insertItem( i18n("&Save configuration"), this,
			SLOT(writeConfiguration()) );
  connect( mOptions, SIGNAL(activated (int)), SLOT( optionMenuCB (int) ));

  //  
  // Define menu entries in the menu bar.
  //
  menuBar()->insertItem( i18n("&File"), mFile );
  menuBar()->insertItem( i18n("&Edit"), mEdit );
  menuBar()->insertItem( i18n("&View"), mView );
  menuBar()->insertItem( i18n("&Tools"), mTools );
  menuBar()->insertItem( i18n("&Documents"), mDocuments );
  menuBar()->insertItem( i18n("&Options"), mOptions );
  menuBar()->insertSeparator();
  menuBar()->insertItem( i18n("&Help"), mHelp );

  
  //
  // Make the toolbar
  //
  QString pixmapPath;
  KIconLoader &loader = *kapp->getIconLoader();
  QPixmap pixmap;

  pixmap = loader.loadIcon( "filenew.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(newFile()), true,
			    i18n("New file") );
  pixmap = loader.loadIcon( "fileopen.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(open()), true,
			    i18n("Open file") );
  pixmap = loader.loadIcon( "reload.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(reload()), true,
			    i18n("Reload file") );
  pixmap = loader.loadIcon( "filefloppy.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(save()), true,
			    i18n("Save file") );
  pixmap = loader.loadIcon( "fileprint.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(print()), true,
			    i18n("Print file") );
  pixmap = loader.loadIcon( "search.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(find()), true,
			    i18n("Find") );
  pixmap = loader.loadIcon( "editcopy.xpm" );
  toolBar(0)->insertButton( pixmap, toolbarCopy, SIGNAL(clicked()),
			    editor(), SLOT(copy()), true,
			    i18n("Copy") );
  pixmap = loader.loadIcon( "editpaste.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(paste()), true,
			    i18n("Paste") );
  pixmap = loader.loadIcon( "editcut.xpm" );
  toolBar(0)->insertButton( pixmap, toolbarCut, SIGNAL(clicked()),
			    editor(), SLOT(cut()), true,
			    i18n("Cut") );
  pixmap = loader.loadIcon( "undo.xpm" );
  toolBar(0)->insertButton( pixmap, toolbarUndo, SIGNAL(clicked()),
			    editor(), SLOT(undo()), true,
			    i18n("Undo") );
  pixmap = loader.loadIcon( "redo.xpm" );
  toolBar(0)->insertButton( pixmap, toolbarRedo, SIGNAL(clicked()),
			    editor(), SLOT(redo()), true,
			    i18n("Redo") );
  pixmap = loader.loadIcon( "flag.xpm" );
  toolBar(0)->insertButton( pixmap, 0, SIGNAL(clicked()),
			    editor(), SLOT(benchmark()), true,
			    i18n("Benchmark") );
  pixmap = loader.loadIcon( "stop.xpm" );
  toolBar(0)->insertButton( pixmap, toolbarInternetStop, SIGNAL(clicked()),
			    editor(), SLOT(stop()), false,
			    i18n("Stop internet connection") );

  //
  // Prepare the status bar. It will display (from right) the offset
  // in the file, Save state ( a pixmap), Insert state, general messages
  //

  statusBar()->setInsertOrder( KStatusBar::RightToLeft );
  statusBar()->insertItem( "Bit: F", 0 );
  statusBar()->insertItem( "Offset: FFFFFFFFFF", 1 );
  statusBar()->insertItem( "Size: FFFFFFFFFF", 2 );
  statusBar()->insertItem( "OVR", 3 );
  statusBar()->insertItem( "M", 4 );
  statusBar()->insertItem( "", 5 );

  statusBar()->setAlignment( 3, AlignCenter );
  statusBar()->setAlignment( 4, AlignCenter );
  statusBar()->changeItem( "Bit:", 0 );
  statusBar()->changeItem( "Offset:", 1 );
  statusBar()->changeItem( "Size:", 2 );
  statusBar()->changeItem( "", 4 );
  //enableStatusBar();

  readConfiguration();
  initialize();

  /*
  mUndoState = UNDO_OK; // Reset in function below
  setUndoState( 0 ); 

  mSelectionValid = true;
  setSelectionState( false );

  displayChanged( mDisplayState );
  mOptions->setItemChecked( mOptionToolBarMB, mShowToolBar );
  mOptions->setItemChecked( mOptionStatusBarMB, mShowStatusBar );
  mOptions->setItemChecked( mOptionUtilityMB, mShowUtility );

  setGeometry( 0, 0, editor()->defaultTextWidth(), 500 );
  */

  mValid = true;
}


KHexEdit::~KHexEdit()
{
  delete mManager;
}


void KHexEdit::initialize( void )
{
  mUndoState = UNDO_OK;   // Reset in function below
  setUndoState( 0 ); 

  mSelectionValid = true; // Reset in function below
  setSelectionState( false );

  textView()->setDisplayState( mDisplayState );
 
  mOptions->setItemChecked( mOptionToolBarMB, mShowToolBar );
  if( mShowToolBar == true )
  {
    enableToolBar(KToolBar::Show);
  }
  else
  {
    enableToolBar(KToolBar::Hide);
  }

  mOptions->setItemChecked( mOptionStatusBarMB, mShowStatusBar );
  if( mShowStatusBar == true )
  {
    enableStatusBar(KStatusBar::Show);
  }
  else
  {
    enableStatusBar(KStatusBar::Hide);
  }

  mOptions->setItemChecked( mOptionUtilityMB, mShowUtility );
  mManager->setShowUtil( mShowUtility );

  setGeometry( 0, 0, editor()->defaultTextWidth(), 500 );
}





void KHexEdit::open( QStrList &fileList )
{
  //
  // We load in reverse to allow first document in the list to
  // become the active one. 
  //
  for( char *s = fileList.last(); s != 0; s = fileList.prev() )
  {
    QString url( s );
    editor()->open( url );
  }
}

void KHexEdit::addRecentFile( QString &fileName )
{
  if( fileName.isEmpty() == true )
  {
    return;
  }
  
  if( fileName.contains( i18n( "Untitled" ), false ) )
  {
    return;
  }

  uint recentCount = mRecent->count();
  uint i;

  for( i=0; i < recentCount; i++ )
  {
    const char *text = mRecent->text(i);
    if( text == 0 )
    {
      continue;
    }
 
    if( fileName == text )
    { 
      mRecent->removeItemAt(i);
    }
  }
  
  if( mRecent->count() > 5 ) 
  {
    mRecent->removeItemAt( 5 );
  }

  mRecent->insertItem( fileName, 0, 0 );
  for( i = 0; i < 5; i++ ) 
  {
    mRecent->setId( i, i );
  }

}



void KHexEdit::recentFile( int id )
{
  const char *text = mRecent->text(id);
  if( text != 0 )
  {
    QString url( text );
    editor()->open( url );
  }
}


void KHexEdit::newWindow( void )
{
  KHexEdit *app = new KHexEdit;
  if( app == 0 || app->isValid() == 0 )
  {
    QString msg;
    msg += i18n( "Can not create new window." );
    msg += "\n";
    msg += i18n( "Insufficient memory." );

    QMessageBox::critical(
    topLevelWidget(),
    i18n("New window" ),
    msg.data(),
    i18n( "OK" ),
    "",
    "",
    0,0);
    return;
  }
  app->show();
}



void KHexEdit::pasteNewWindow( void )
{
  KHexEdit *app = new KHexEdit;
  if( app == 0 || app->isValid() == 0 )
  {
    QString msg;
    msg += i18n( "Can not create new window." );
    msg += "\n";
    msg += i18n( "Insufficient memory." );

    QMessageBox::critical(
    topLevelWidget(),
    i18n("New window" ),
    msg.data(),
    i18n( "OK" ),
    "",
    "",
    0,0);
    return;
  }
  app->show();
  app->editor()->pasteNewFile();
}











void KHexEdit::closeWindow( void )
{
  close();
}


void KHexEdit::closeProgram( void )
{
  if( mWindowList.count() > 1 )
  {
    for( KHexEdit *w = mWindowList.first(); w != 0; w = mWindowList.next() )
    {
      if( w->editor()->modified() == true )
      {
	QString msg;
	msg += i18n( "There are windows with unsaved modified documents." );
	msg += "\n";
	msg += i18n( "If you quit now, these modifications will be lost." );
	msg += "\n";
	msg += i18n( "Quit anyways?" );
      
	int reply = QMessageBox::warning( 
	  topLevelWidget(),
	  i18n( "Warning" ), 
	  msg.data(),
	  i18n( "&Yes" ),
	  i18n( "&No" ),
	  "",
	  1, 1 );
	if( reply == 0 ) // Yes
	{
	  break;
	}
	else
	{
	  return;
	}
      }
    }
    queryExit(); // Just to save some data
    kapp->quit();
  }
  else
  {
    closeWindow();
  }
}
    

bool KHexEdit::queryClose( void )
{
  bool success = editor()->closeAll();
  if( success == true )
  {
    mWindowList.removeRef( this );
  }

  return( success );
}


//
// We will always return true here. 
//
bool KHexEdit::queryExit( void )
{
  writeConfiguration();
  return( true );
}



void KHexEdit::readConfiguration( void )
{
  KConfig &config = *kapp->getConfig();
  readConfiguration( config );
}


void KHexEdit::writeConfiguration( void )
{
  KConfig &config = *kapp->getConfig();
  writeConfiguration( config );
}


void KHexEdit::writeConfiguration( KConfig &config )
{
  config.setGroup("General Options" );
  config.writeEntry("Width", width() );
  config.writeEntry("Height", height() );
  config.writeEntry("ShowToolBar", mShowToolBar );
  config.writeEntry("ShowStatusBar", mShowStatusBar );
  config.writeEntry("ShowUtility", mShowUtility );

  for( uint i=0; i < mRecent->count(); i++ )
  {
    const char *text = mRecent->text(i);
    if( text != 0 )
    {
      char buf[20];
      sprintf( buf, "Recent%d", i+1 );
      config.writeEntry( buf, text );
    }
  }

  config.setGroup("Display Options" );
  config.writeEntry("ViewMode", mDisplayState.viewModeString() );
  config.writeEntry("TextMode", mDisplayState.textModeString() );
  config.writeEntry("OffsetMode", mDisplayState.offsetModeString() );
  config.writeEntry("ViewCaps", mDisplayState.viewCaps );
  config.writeEntry("OffsetCaps", mDisplayState.offsetCaps );
  config.writeEntry("LineSize", mDisplayState.lineSize );
  config.writeEntry("GroupSize", mDisplayState.groupSize );
  config.writeEntry("SplitGroup", mDisplayState.splitGroup );
  config.writeEntry("WrapGroup", mDisplayState.wrapInGroup );
  config.writeEntry("SplitWidth", mDisplayState.splitWidth );
}


void KHexEdit::readConfiguration( KConfig &config )
{
  config.setGroup("General Options");
  mShowToolBar   = config.readBoolEntry( "ShowToolBar", true );
  mShowStatusBar = config.readBoolEntry( "ShowStatusBar", true );
  mShowUtility   = config.readBoolEntry( "ShowUtility", true );
  for( uint i = 0; i < 5; i++ ) 
  {
    char buf[20];
    sprintf( buf, "Recent%d", i + 1 );
    QString fileName = config.readEntry( buf );
    if( fileName.isEmpty() == false )
    { 
      mRecent->insertItem( fileName );
    }
  }

  config.setGroup("Display Options" );
  mDisplayState.setViewMode( config.readEntry("ViewMode") );
  mDisplayState.setTextMode( config.readEntry("TextMode") );
  mDisplayState.setOffsetMode( config.readEntry("OffsetMode") );
  mDisplayState.viewCaps = config.readBoolEntry( "ViewCaps", false );
  mDisplayState.offsetCaps = config.readBoolEntry( "OffsetCaps", false );
  mDisplayState.lineSize = config.readNumEntry( "LineSize", 16 );
  mDisplayState.groupSize = config.readNumEntry("GroupSize", 1 );
  mDisplayState.splitGroup = config.readBoolEntry("SplitGroup", true );
  mDisplayState.wrapInGroup = config.readBoolEntry("WrapGroup", false );
  mDisplayState.splitWidth = config.readNumEntry( "SplitWidth", -1 );
}



void KHexEdit::internetLoadingChanged( bool state )
{
  toolBar()->setItemEnabled( toolbarInternetStop, state );
}




void KHexEdit::cursorChanged( SCursorState &state )
{
  if( state.valid == true )
  {
    char buf[20];
    sprintf( buf, "Bit: %u", state.cell );
    statusBar()->changeItem( buf, 0 );
    if( mShowHexOffset == true )
    {
      if( mView->isItemChecked( mViewBigCaseOffsetMB ) )
      {
	sprintf( buf, "Offset: %04X:%04X", state.offset>>16, 
		 state.offset&0x0000FFFF );
      }
      else
      {
	sprintf( buf, "Offset: %04x:%04x", state.offset>>16, 
		 state.offset&0x0000FFFF );
      }
    }
    else
    {
      sprintf( buf, "Offset: %010u", state.offset );
    }
    statusBar()->changeItem( buf, 1 );
  }
  else
  {
    statusBar()->changeItem( "Bit:", 0 );
    statusBar()->changeItem( "Offset:", 1 );
  }
  
  setUndoState( state.undoState );
  setSelectionState( state.selectionValid );
}


void KHexEdit::fileModified( SFileState &state )
{
  if( state.valid == true )
  {
    char buf[20];
    sprintf( buf, "Size: %u", state.size );
    statusBar()->changeItem( buf, 2 );
    statusBar()->changeItem( state.modified == true ? "!" : "", 4 );
  }
  else
  {
    statusBar()->changeItem( "", 2 );
    statusBar()->changeItem( "", 4 );
  }
}



void KHexEdit::displayChanged( const SDisplayState &state )
{
  if( state.viewMode == SDisplayState::view_hexadecimal ) 
  {
    mView->setItemChecked( mViewHexMB, true );
    mView->setItemEnabled( mViewBigCaseDataMB, true );
  }
  else
  {
    mView->setItemChecked( mViewHexMB, false );
    mView->setItemEnabled( mViewBigCaseDataMB, false );
  }

  if( state.viewMode == SDisplayState::view_octal )
  { 
    mView->setItemChecked( mViewOctalMB, true );
  }
  else
  {
    mView->setItemChecked( mViewOctalMB, false );
  }

  if( state.viewMode == SDisplayState::view_binary )
  { 
    mView->setItemChecked( mViewBinaryMB, true );
  }
  else
  {
    mView->setItemChecked( mViewBinaryMB, false );
  }
  
  if( state.viewMode == SDisplayState::view_ascii )
  { 
    mView->setItemChecked( mViewTextOnlyMB, true );
    mView->setItemEnabled( mViewTextAreaMB, false );
  }
  else
  {
    mView->setItemChecked( mViewTextOnlyMB, false );
    mView->setItemEnabled( mViewTextAreaMB, true );
  }

  if( state.offsetMode == SDisplayState::offset_hide )
  {
    mView->setItemChecked( mViewOffsetColumnMB, false );
    mView->setItemEnabled( mViewBigCaseOffsetMB, false );
  }
  else
  {
    mView->setItemChecked( mViewOffsetColumnMB, true );
    mView->setItemEnabled( mViewBigCaseOffsetMB, true );
  }

  if( state.textMode == SDisplayState::text_hide )
  {
    mView->setItemChecked( mViewTextAreaMB, false );
  }
  else
  {
    mView->setItemChecked( mViewTextAreaMB, true );
  }

  mView->setItemChecked( mViewBigCaseOffsetMB, state.offsetCaps );
  mView->setItemChecked( mViewBigCaseDataMB, state.viewCaps );

  mDisplayState = state;
}



void KHexEdit::fileActive( QString &url )
{
  QString str( kapp->getCaption() );
  if( url.isNull() == false )
  {
    str += ": ";
    str += url;
    addDocument( url );
    addRecentFile( url );
  }

  setCaption( str.data() );
}

void KHexEdit::fileClosed( QString &url )
{
  removeDocument( url );
}



void KHexEdit::editMode( CHexBuffer::EEditMode editMode )
{
  statusBar()->changeItem( editMode == CHexBuffer::edit_overwrite ? 
			   i18n("OVR") : i18n("INS"), 3 );
}


void KHexEdit::textWidth( uint width )
{
  setGeometry( x(), y(), editor()->defaultTextWidth(), height() );
}



void KHexEdit::dropAction( KDNDDropZone *dropZone )
{
  QStrList &list = dropZone->getURLList();
  open( list );
}


void KHexEdit::viewMenuCB( int id )
{
  switch( id )
  {
    case Xview_hexadecimal:
      mDisplayState.viewMode      = SDisplayState::view_hexadecimal;
      mDisplayState.lineSize      = 16;
      mDisplayState.groupSize     = 1;
      mDisplayState.splitWidth    = -1;
      mDisplayState.splitGroup    = true;
      mDisplayState.wrapInGroup   = true;
      mDisplayState.textMode      = SDisplayState::text_ascii;
      textView()->setDisplayState( mDisplayState );
    break;

    case Xview_octal:
      mDisplayState.viewMode      = SDisplayState::view_octal;
      mDisplayState.lineSize      = 16;
      mDisplayState.groupSize     = 1;
      mDisplayState.splitWidth    = -1;
      mDisplayState.splitGroup    = true;
      mDisplayState.wrapInGroup   = true;
      mDisplayState.textMode      = SDisplayState::text_ascii;
      textView()->setDisplayState( mDisplayState );
    break;

    case Xview_binary:
      mDisplayState.viewMode      = SDisplayState::view_binary;
      mDisplayState.lineSize      = 8;
      mDisplayState.groupSize     = 1;
      mDisplayState.splitWidth    = -1;
      mDisplayState.splitGroup    = true;
      mDisplayState.wrapInGroup   = true;
      mDisplayState.textMode      = SDisplayState::text_ascii;
      textView()->setDisplayState( mDisplayState );
    break;

    case Xview_text:
      mDisplayState.viewMode      = SDisplayState::view_ascii;
      mDisplayState.lineSize      = 64;
      mDisplayState.groupSize     = 64;
      mDisplayState.splitWidth    = -1;
      mDisplayState.splitGroup    = false;
      mDisplayState.wrapInGroup   = true;
      mDisplayState.textMode      = SDisplayState::text_hide;
      textView()->setDisplayState( mDisplayState );
    break;

    case Xview_offset_area:
      if( mDisplayState.offsetMode == SDisplayState::offset_hide )
      {
	mDisplayState.offsetMode = SDisplayState::offset_hexadecimal;
      }
      else
      {
	mDisplayState.offsetMode = SDisplayState::offset_hide;
      }  
      textView()->setDisplayState( mDisplayState );
    break;

    case Xview_text_area:
      if( mDisplayState.textMode == SDisplayState::text_hide )
      {
	mDisplayState.textMode = SDisplayState::text_ascii;
      }
      else
      {
	mDisplayState.textMode = SDisplayState::text_hide;
      }
      textView()->setDisplayState( mDisplayState );
    break;

    case Xview_offset_as_decimal:
    break;

    case Xview_case_data:
      if( mView->isItemChecked( mViewBigCaseDataMB ) == true )
      {	
	mDisplayState.viewCaps = false;
      }
      else
      {
	mDisplayState.viewCaps = true;
      }
      textView()->setDisplayState( mDisplayState );
    break;

    case Xview_case_offset:
      if( mView->isItemChecked( mViewBigCaseOffsetMB ) == true )
      {
	mDisplayState.offsetCaps = false;
      }
      else
      {
	mDisplayState.offsetCaps = true;
      }
      textView()->setDisplayState( mDisplayState );
    break;
  }
}



void KHexEdit::optionMenuCB( int id )
{
  switch( id )
  {
    case option_toolbar:
      mShowToolBar = mShowToolBar == true ? false : true;
      mOptions->setItemChecked( mOptionToolBarMB, mShowToolBar );
      if( mShowToolBar == true )
      {
	enableToolBar(KToolBar::Show);
      }
      else
      {
	enableToolBar(KToolBar::Hide);
      }
    break;

    case option_statusbar:
      mShowStatusBar = mShowStatusBar == true ? false : true;
      mOptions->setItemChecked( mOptionStatusBarMB, mShowStatusBar );
      if( mShowStatusBar == true )
      {
	enableStatusBar(KStatusBar::Show);
      }
      else
      {
	enableStatusBar(KStatusBar::Hide);
      }
    break;

    case option_utility:
      mShowUtility = mManager->showUtil() == true ? false : true;
      mOptions->setItemChecked( mOptionUtilityMB, mShowUtility );
      mManager->setShowUtil( mShowUtility );
    break;

    case option_edit:
      editor()->options();
    break;
  }
}





void KHexEdit::setUndoState( uint undoState ) 
{
  if( mUndoState != undoState )
  {
    mUndoState = undoState;
    mEdit->setItemEnabled( mEditUndoMB, mUndoState & UNDO_OK );
    mEdit->setItemEnabled( mEditRedoMB, mUndoState & REDO_OK );
    toolBar()->setItemEnabled( toolbarUndo, mUndoState & UNDO_OK );
    toolBar()->setItemEnabled( toolbarRedo, mUndoState & REDO_OK );
  }
}


void KHexEdit::setSelectionState( bool selectionValid )
{
  if( mSelectionValid != selectionValid )
  {
    mSelectionValid = selectionValid;
    mEdit->setItemEnabled( mEditCopyMB, mSelectionValid );
    mEdit->setItemEnabled( mEditCutMB, mSelectionValid );
    mEditSpecial->setItemEnabled( mEditSpecialCopyText1MB, mSelectionValid );
    mEditSpecial->setItemEnabled( mEditSpecialCopyText2MB, mSelectionValid );

    toolBar()->setItemEnabled( toolbarCopy, mSelectionValid );
    toolBar()->setItemEnabled( toolbarCut, mSelectionValid );
  }
}



void KHexEdit::selectDocument( int id )
{
  const char *text = mDocuments->text(id);
  if( text != 0 )
  {
    QString url( text );
    editor()->selectDocument( url );
  }

}



void KHexEdit::addDocument( QString &fileName )
{
  QString *str;
  for( str = mDocumentList.first(); str != 0; str = mDocumentList.next() )
  {
    if( *str == fileName )
    {
      return;
    }
  }

  str = new QString( fileName.data() );
  if( str == 0 )
  {
    return;
  }

  mDocumentList.append( str );
  mDocuments->insertItem( fileName );
}



void KHexEdit::removeDocument( QString &fileName )
{
  QString *str;
  for( str = mDocumentList.first(); str != 0; str = mDocumentList.next() )
  {
    if( *str == fileName )
    {
      mDocumentList.remove();
      mDocuments->clear();

      for( str = mDocumentList.first(); str != 0; str = mDocumentList.next() )
      {
	mDocuments->insertItem( *str );
      }
      break;
    }
  }
}




