#ifndef _TOPLEVEL_H_
#define _TOPLEVEL_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <kapp.h>
#include <ktmainwindow.h>

#include "hexbuffer.h"
#include "hexmanagerwidget.h"

class KHexEdit : public KTMainWindow
{
  Q_OBJECT

  public:
    KHexEdit( void );
    ~KHexEdit( void );

    inline bool isValid( void );
    void open( QStrList &fileList );
    void initialize( void );

  public slots:
    void dropAction( KDNDDropZone *zone );
    void recentFile( int id );
    void newWindow( void );
    void pasteNewWindow( void );
    void closeWindow( void );
    void closeProgram( void );
    void internetLoadingChanged( bool state );
    void selectDocument( int id );

    void cursorChanged( SCursorState &state );
    void fileModified( SFileState &state );
    void displayChanged( const SDisplayState &state );
    void fileActive( QString &url );
    void fileClosed( QString &url );

    void readConfiguration( void );
    void writeConfiguration( void );

    void editMode( CHexBuffer::EEditMode editMode );
    void textWidth( uint width );
    void viewMenuCB( int id );
    void optionMenuCB( int id );

  protected:
    virtual bool queryExit( void );
    virtual bool queryClose( void );
   
  private:
    void addRecentFile( QString &fileName );
    bool closeAllWindow( void );
    void setUndoState( uint undoState );
    void setSelectionState( bool selectionValid );
    void addDocument( QString &fileName );
    void removeDocument( QString &fileName );

    void writeConfiguration( KConfig &config );
    void readConfiguration( KConfig &config );

    inline CHexEditorWidget *editor( void );
    inline CHexViewWidget *textView( void );
 
  private:
    static QList<KHexEdit> mWindowList;
    CHexManagerWidget *mManager;
    KDNDDropZone *mZone;

    QList<QString> mDocumentList;

    QPopupMenu *mFile;
    QPopupMenu *mRecent;
    QPopupMenu *mEdit;
    QPopupMenu *mEditSpecial;
    QPopupMenu *mView;
    QPopupMenu *mDocuments;
    QPopupMenu *mTools;
    QPopupMenu *mOptions;
    QPopupMenu *mHelp;

    int mEditUndoMB, mEditRedoMB, mEditCopyMB, mEditCutMB;
    int mEditSpecialCopyText1MB, mEditSpecialCopyText2MB;
    int mViewHexMB, mViewOctalMB, mViewBinaryMB, mViewTextOnlyMB;
    int mViewOffsetColumnMB, mViewTextAreaMB, mViewOffsetAsDecimalMB;
    int mViewBigCaseDataMB, mViewBigCaseOffsetMB;
    int mOptionToolBarMB, mOptionStatusBarMB, mOptionUtilityMB;

    bool mValid;
    SDisplayState mDisplayState;
    bool mShowHexOffset;
    bool mShowToolBar;
    bool mShowStatusBar;
    bool mShowUtility;
    bool mSelectionValid;
    uint mUndoState;
    uint mViewMode;

};


inline bool KHexEdit::isValid( void )
{
  return( mValid );
}

inline CHexEditorWidget *KHexEdit::editor( void )
{
  return( mManager->editor() );
}

inline CHexViewWidget *KHexEdit::textView( void )
{
  return( mManager->editor()->view() );
}



#endif


